@@ -81,7 +81,7 @@ Value* getConstantValue(Type* T)
     case Type::StructTyID:    case Type::ArrayTyID:
         return Constant::getNullValue(T);
     default:
-        assert(0); exit(1);
+        assert(0); exit(RET_ERROR);
     }
 }
 
@@ -420,7 +420,7 @@ bool iterInstComb(std::fstream &listf, Instruction *SIclone, Instruction *SI, un
         }
     }
     else {
-        assert(0); exit(1);
+        assert(0); exit(RET_ERROR);
     }
 }
 
@@ -780,5 +780,5 @@ unsigned int getAlignFromType(Module &M, Type *T)
     else if (T == Type::getDoublePtrTy(M.getContext()))
         return 8;
     else
-        assert(0); exit(1);
+        assert(0); exit(RET_ERROR);;
 }
\ No newline at end of file
